/*
 * This file is part of John the Ripper password cracker,
 * Copyright (c) 1996-99 by Solar Designer
 */

#ifndef vms
#ifndef __DJGPP__
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#else
#include <bios.h>
#endif

#if !defined(O_NONBLOCK) && defined(O_NDELAY)
#define O_NONBLOCK			O_NDELAY
#endif

#ifdef __CYGWIN32__
#include <string.h>
#include <sys/socket.h>
#ifndef __CYGWIN__
extern int tcgetattr(int fd, struct termios *termios_p);
extern int tcsetattr(int fd, int actions, struct termios *termios_p);
#endif
#endif

#include "tty.h"

#ifndef __DJGPP__
static int tty_fd = 0;
static struct termios saved_ti;
#endif

void tty_init(void)
{
#ifndef __DJGPP__
	int fd;
	struct termios ti;

	if (tty_fd) return;

	if ((fd = open("/dev/tty", O_RDONLY | O_NONBLOCK)) < 0) return;

#ifndef __CYGWIN32__
	if (tcgetpgrp(fd) != getpid()) {
		close(fd); return;
	}
#endif

	tcgetattr(fd, &ti);
	saved_ti = ti;
	ti.c_lflag &= ~(ICANON | ECHO);
	ti.c_cc[VMIN] = 1;
	ti.c_cc[VTIME] = 0;
	tcsetattr(fd, TCSANOW, &ti);

	tty_fd = fd;

	atexit(tty_done);
#endif
}

int tty_getchar(void)
{
#ifndef __DJGPP__
	int c;
#ifdef __CYGWIN32__
	fd_set set;
	struct timeval tv;
#endif

	if (tty_fd) {
#ifdef __CYGWIN32__
		FD_ZERO(&set); FD_SET(tty_fd, &set);
		tv.tv_sec = 0; tv.tv_usec = 0;
		if (select(tty_fd + 1, &set, NULL, NULL, &tv) <= 0)
			return -1;
#endif
		c = 0;
		if (read(tty_fd, &c, 1) > 0) return c;
	}
#else
	if (_bios_keybrd(_KEYBRD_READY))
		return _bios_keybrd(_KEYBRD_READ);
#endif

	return -1;
}

void tty_done(void)
{
#ifndef __DJGPP__
	int fd;

	if (!tty_fd) return;

	fd = tty_fd; tty_fd = 0;
	tcsetattr(fd, TCSANOW, &saved_ti);

	close(fd);
#endif
}

#else
/* VMS code provided by Martin Vorlaender. This code is so different that
 * we separate it otherwise the whole file becomes completely unreadable.
 */
#include <descrip.h>
#include <stsdef.h>
#include <iodef.h>
#include <starlet.h>
#include <stdlib.h>

#include "tty.h"

static unsigned short chan = 0;

static volatile struct {
	unsigned short sts;
	unsigned short count;
	unsigned long dev_specific;
} iosb;

static volatile int c;

static int queue_read(void)
{
	return sys$qio( 0L, chan, IO$_READVBLK|IO$M_NOECHO, &iosb, NULL, 0L,
		&c, 1, 0, NULL, NULL, 0);
}

void tty_init(void)
{
	$DESCRIPTOR(dsc_tt, "TT:");
	int sts;

	if (chan) return;

	sts = sys$assign(&dsc_tt, &chan, 0L, NULL, 0L);
	if (! $VMS_STATUS_SUCCESS(sts)) return;

	sts = queue_read();
	if (! $VMS_STATUS_SUCCESS(sts)) {
		int sts2 = sys$dassgn(chan);
		return;
	}
	atexit(tty_done);
}

int tty_getchar(void)
{
	int sts;
	int saved_c;

	if (chan &&
	    iosb.sts != 0 && $VMS_STATUS_SUCCESS(iosb.sts) && iosb.count > 0) {
		saved_c = c;
		sts = queue_read();
		return saved_c;
	}
	return -1;
}

void tty_done(void)
{
	if (!chan) return;
	(void)sys$dassgn(chan);
}
#endif /* vms */
